'''
Dict-style key value wrapper around some embeded databases
==========================================================

Usage
=====

    >>> import unidbm
    >>> db = unidbm.open('sqlite', path=':memory:')

Set and get data:

    >>> db[u'foo'] = ['bar', {'baz': 1}]
    >>> db[u'foo']
    ['bar', {'baz': 1}]

Iterate:

    >>> [key for key in db] == [u'foo']
    True

Delete:

    >>> del db[u'foo']
    >>> len(db)
    0

Backends
--------
- sqlite
- kyoto cabinet
- semidbm

Custom backend
--------------
Backend works with bytes (str in py2) keys and values.
It should implement next methods:

- __init__(self, path, **any_options)
- def __getitem__(self, key):
- def __setitem__(self, key, value):
- def __len__(self):
- def __iter__(self):
- def close(self):


'''
from __future__ import absolute_import
import sys
try:
    from UserDict import DictMixin
except ImportError:
    from collections import MutableMapping as DictMixin

from . import backends
from . import middlewares
from .utils import name_to_object


__version__ = '0.1.0'


if sys.version_info < (3, ):
    bytes = str
else:
    unicode = str


class DBM(DictMixin):
    def __init__(self, backend, middlewares=None):
        middlewares = middlewares or []
        self.backend = backend
        self.set_middlewares = middlewares
        self.get_middlewares = self.set_middlewares[::-1]

    def __getitem__(self, key):
        key = self._encode_key(key)
        value = self.backend[key]
        for middleware in self.get_middlewares:
            value = middleware.load(value)
        return value

    def __setitem__(self, key, value):
        key = self._encode_key(key)
        for middleware in self.set_middlewares:
            value = middleware.dump(value)
        self.backend[key] = value

    def __delitem__(self, key):
        key = self._encode_key(key)
        del self.backend[key]

    def __len__(self):
        return len(self.backend)

    def __iter__(self):
        return (k.decode('utf-8') for k in self.backend)

    @staticmethod
    def _encode_key(key):
        if not isinstance(key, unicode):
            raise TypeError('unidbm key should be an unicode string')
        return key.encode('utf-8')


def open(backend, serialize=True, dumper='pickle',
         compress=False, compress_level=9,
         **backend_kwargs):
    mdls = []
    if serialize:
        mdls.append(get_dumper(dumper)())
    if compress:
        middleware = name_to_object(middlewares.NAMES['compress'])
        mdls.append(middleware(compress_level=compress_level))
    backend = get_backend(backend)(**backend_kwargs)
    return DBM(backend, middlewares=mdls)


def get_dumper(name):
    name = middlewares.NAMES.get(name, name)
    try:
        return name_to_object(name)
    except ImportError:
        raise ImportError('Unknown dumper: {}'.format(name))


def get_backend(name):
    name = backends.NAMES.get(name, name)
    try:
        return name_to_object(name)
    except ImportError:
        raise ImportError('Unknown backend: {}'.format(name))



#!/usr/bin/env python
import os
import sys
import doctest
from codecs import open

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import unidbm as module

name = module.__name__
readme = module.__doc__.strip()

with open('README.md', 'w', 'utf-8') as f:
    f.write(readme)
if doctest.testfile('README.md',
                    optionflags=doctest.REPORT_ONLY_FIRST_FAILURE).failed:
    sys.exit(1)

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit(0)

setup(
    name=module.__name__,
    version=module.__version__,
    description=readme.splitlines()[0],
    long_description=readme,
    author='Imbolc',
    author_email='imbolc@imbolc.name',
    url='https://github.com/imbolc/%s' % name,
    packages=[name],
    install_requires=[],
    license='ISC',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4'
    ),
)
